# Generated by the protocol buffer compiler.  DO NOT EDIT!
# sources: cosmos/vesting/v1beta1/tx.proto, cosmos/vesting/v1beta1/vesting.proto
# plugin: python-betterproto
# This file has been @generated

from dataclasses import dataclass
from typing import (
    TYPE_CHECKING,
    Dict,
    List,
    Optional,
)

import betterproto
import grpclib
from betterproto.grpc.grpclib_server import ServiceBase

from ...auth import v1beta1 as __auth_v1_beta1__
from ...base import v1beta1 as __base_v1_beta1__


if TYPE_CHECKING:
    import grpclib.server
    from betterproto.grpc.grpclib_client import MetadataLike
    from grpclib.metadata import Deadline


@dataclass(eq=False, repr=False)
class BaseVestingAccount(betterproto.Message):
    """
    BaseVestingAccount implements the VestingAccount interface. It contains all
     the necessary fields needed for any vesting account implementation.
    """

    base_account: "__auth_v1_beta1__.BaseAccount" = betterproto.message_field(1)
    original_vesting: List["__base_v1_beta1__.Coin"] = betterproto.message_field(2)
    delegated_free: List["__base_v1_beta1__.Coin"] = betterproto.message_field(3)
    delegated_vesting: List["__base_v1_beta1__.Coin"] = betterproto.message_field(4)
    end_time: int = betterproto.int64_field(5)


@dataclass(eq=False, repr=False)
class ContinuousVestingAccount(betterproto.Message):
    """
    ContinuousVestingAccount implements the VestingAccount interface. It
     continuously vests by unlocking coins linearly with respect to time.
    """

    base_vesting_account: "BaseVestingAccount" = betterproto.message_field(1)
    start_time: int = betterproto.int64_field(2)


@dataclass(eq=False, repr=False)
class DelayedVestingAccount(betterproto.Message):
    """
    DelayedVestingAccount implements the VestingAccount interface. It vests all
     coins after a specific time, but non prior. In other words, it keeps them
     locked until a specified time.
    """

    base_vesting_account: "BaseVestingAccount" = betterproto.message_field(1)


@dataclass(eq=False, repr=False)
class Period(betterproto.Message):
    """Period defines a length of time and amount of coins that will vest."""

    length: int = betterproto.int64_field(1)
    amount: List["__base_v1_beta1__.Coin"] = betterproto.message_field(2)


@dataclass(eq=False, repr=False)
class PeriodicVestingAccount(betterproto.Message):
    """
    PeriodicVestingAccount implements the VestingAccount interface. It
     periodically vests by unlocking coins during each specified period.
    """

    base_vesting_account: "BaseVestingAccount" = betterproto.message_field(1)
    start_time: int = betterproto.int64_field(2)
    vesting_periods: List["Period"] = betterproto.message_field(3)


@dataclass(eq=False, repr=False)
class PermanentLockedAccount(betterproto.Message):
    """
    PermanentLockedAccount implements the VestingAccount interface. It does
     not ever release coins, locking them indefinitely. Coins in this account can
     still be used for delegating and for governance votes even while locked.

     Since: cosmos-sdk 0.43
    """

    base_vesting_account: "BaseVestingAccount" = betterproto.message_field(1)


@dataclass(eq=False, repr=False)
class MsgCreateVestingAccount(betterproto.Message):
    """
    MsgCreateVestingAccount defines a message that enables creating a vesting
     account.
    """

    from_address: str = betterproto.string_field(1)
    to_address: str = betterproto.string_field(2)
    amount: List["__base_v1_beta1__.Coin"] = betterproto.message_field(3)
    end_time: int = betterproto.int64_field(4)
    delayed: bool = betterproto.bool_field(5)


@dataclass(eq=False, repr=False)
class MsgCreateVestingAccountResponse(betterproto.Message):
    """
    MsgCreateVestingAccountResponse defines the Msg/CreateVestingAccount response type.
    """

    pass


@dataclass(eq=False, repr=False)
class MsgCreatePeriodicVestingAccount(betterproto.Message):
    """
    MsgCreatePeriodicVestingAccount defines a message that enables creating a vesting
     account.
    """

    from_address: str = betterproto.string_field(1)
    to_address: str = betterproto.string_field(2)
    start_time: int = betterproto.int64_field(3)
    vesting_periods: List["Period"] = betterproto.message_field(4)


@dataclass(eq=False, repr=False)
class MsgCreatePeriodicVestingAccountResponse(betterproto.Message):
    """
    MsgCreatePeriodicVestingAccountResponse defines the Msg/CreatePeriodicVestingAccount
     response type.
    """

    pass


@dataclass(eq=False, repr=False)
class MsgDonateAllVestingTokens(betterproto.Message):
    """
    MsgDonateAllVestingTokens defines a message that enables donating all vesting
     token to community pool.
    """

    from_address: str = betterproto.string_field(1)


@dataclass(eq=False, repr=False)
class MsgDonateAllVestingTokensResponse(betterproto.Message):
    """
    MsgDonateAllVestingTokensResponse defines the Msg/MsgDonateAllVestingTokens
     response type.
    """

    pass


class MsgStub(betterproto.ServiceStub):
    async def create_vesting_account(
        self,
        msg_create_vesting_account: "MsgCreateVestingAccount",
        *,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None
    ) -> "MsgCreateVestingAccountResponse":
        return await self._unary_unary(
            "/cosmos.vesting.v1beta1.Msg/CreateVestingAccount",
            msg_create_vesting_account,
            MsgCreateVestingAccountResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    async def create_periodic_vesting_account(
        self,
        msg_create_periodic_vesting_account: "MsgCreatePeriodicVestingAccount",
        *,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None
    ) -> "MsgCreatePeriodicVestingAccountResponse":
        return await self._unary_unary(
            "/cosmos.vesting.v1beta1.Msg/CreatePeriodicVestingAccount",
            msg_create_periodic_vesting_account,
            MsgCreatePeriodicVestingAccountResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    async def donate_all_vesting_tokens(
        self,
        msg_donate_all_vesting_tokens: "MsgDonateAllVestingTokens",
        *,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None
    ) -> "MsgDonateAllVestingTokensResponse":
        return await self._unary_unary(
            "/cosmos.vesting.v1beta1.Msg/DonateAllVestingTokens",
            msg_donate_all_vesting_tokens,
            MsgDonateAllVestingTokensResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )


class MsgBase(ServiceBase):
    async def create_vesting_account(
        self, msg_create_vesting_account: "MsgCreateVestingAccount"
    ) -> "MsgCreateVestingAccountResponse":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def create_periodic_vesting_account(
        self, msg_create_periodic_vesting_account: "MsgCreatePeriodicVestingAccount"
    ) -> "MsgCreatePeriodicVestingAccountResponse":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def donate_all_vesting_tokens(
        self, msg_donate_all_vesting_tokens: "MsgDonateAllVestingTokens"
    ) -> "MsgDonateAllVestingTokensResponse":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def __rpc_create_vesting_account(
        self,
        stream: "grpclib.server.Stream[MsgCreateVestingAccount, MsgCreateVestingAccountResponse]",
    ) -> None:
        request = await stream.recv_message()
        response = await self.create_vesting_account(request)
        await stream.send_message(response)

    async def __rpc_create_periodic_vesting_account(
        self,
        stream: "grpclib.server.Stream[MsgCreatePeriodicVestingAccount, MsgCreatePeriodicVestingAccountResponse]",
    ) -> None:
        request = await stream.recv_message()
        response = await self.create_periodic_vesting_account(request)
        await stream.send_message(response)

    async def __rpc_donate_all_vesting_tokens(
        self,
        stream: "grpclib.server.Stream[MsgDonateAllVestingTokens, MsgDonateAllVestingTokensResponse]",
    ) -> None:
        request = await stream.recv_message()
        response = await self.donate_all_vesting_tokens(request)
        await stream.send_message(response)

    def __mapping__(self) -> Dict[str, grpclib.const.Handler]:
        return {
            "/cosmos.vesting.v1beta1.Msg/CreateVestingAccount": grpclib.const.Handler(
                self.__rpc_create_vesting_account,
                grpclib.const.Cardinality.UNARY_UNARY,
                MsgCreateVestingAccount,
                MsgCreateVestingAccountResponse,
            ),
            "/cosmos.vesting.v1beta1.Msg/CreatePeriodicVestingAccount": grpclib.const.Handler(
                self.__rpc_create_periodic_vesting_account,
                grpclib.const.Cardinality.UNARY_UNARY,
                MsgCreatePeriodicVestingAccount,
                MsgCreatePeriodicVestingAccountResponse,
            ),
            "/cosmos.vesting.v1beta1.Msg/DonateAllVestingTokens": grpclib.const.Handler(
                self.__rpc_donate_all_vesting_tokens,
                grpclib.const.Cardinality.UNARY_UNARY,
                MsgDonateAllVestingTokens,
                MsgDonateAllVestingTokensResponse,
            ),
        }



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# sources: cosmos/base/query/v1beta1/pagination.proto
# plugin: python-betterproto
# This file has been @generated

from dataclasses import dataclass

import betterproto


@dataclass(eq=False, repr=False)
class PageRequest(betterproto.Message):
    """
    PageRequest is to be embedded in gRPC request messages for efficient
     pagination. Ex:

      message SomeRequest {
              Foo some_parameter = 1;
              PageRequest pagination = 2;
      }
    """

    key: bytes = betterproto.bytes_field(1)
    """
    key is a value returned in PageResponse.next_key to begin
     querying the next page most efficiently. Only one of offset or key
     should be set.
    """

    offset: int = betterproto.uint64_field(2)
    """
    offset is a numeric offset that can be used when key is unavailable.
     It is less efficient than using key. Only one of offset or key should
     be set.
    """

    limit: int = betterproto.uint64_field(3)
    """
    limit is the total number of results to be returned in the result page.
     If left empty it will default to a value to be set by each app.
    """

    count_total: bool = betterproto.bool_field(4)
    """
    count_total is set to true  to indicate that the result set should include
     a count of the total number of items available for pagination in UIs.
     count_total is only respected when offset is used. It is ignored when key
     is set.
    """

    reverse: bool = betterproto.bool_field(5)
    """
    reverse is set to true if results are to be returned in the descending order.
    
     Since: cosmos-sdk 0.43
    """


@dataclass(eq=False, repr=False)
class PageResponse(betterproto.Message):
    """
    PageResponse is to be embedded in gRPC response messages where the
     corresponding request message has used PageRequest.

      message SomeResponse {
              repeated Bar results = 1;
              PageResponse page = 2;
      }
    """

    next_key: bytes = betterproto.bytes_field(1)
    """
    next_key is the key to be passed to PageRequest.key to
     query the next page most efficiently
    """

    total: int = betterproto.uint64_field(2)
    """
    total is total number of results available if PageRequest.count_total
     was set, its value is undefined otherwise
    """



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# sources: ethermint/feemarket/v1/feemarket.proto, ethermint/feemarket/v1/genesis.proto, ethermint/feemarket/v1/query.proto
# plugin: python-betterproto
# This file has been @generated

from dataclasses import dataclass
from typing import (
    TYPE_CHECKING,
    Dict,
    Optional,
)

import betterproto
import grpclib
from betterproto.grpc.grpclib_server import ServiceBase


if TYPE_CHECKING:
    import grpclib.server
    from betterproto.grpc.grpclib_client import MetadataLike
    from grpclib.metadata import Deadline


@dataclass(eq=False, repr=False)
class Params(betterproto.Message):
    """Params defines the EVM module parameters"""

    no_base_fee: bool = betterproto.bool_field(1)
    """
    no base fee forces the EIP-1559 base fee to 0 (needed for 0 price calls)
    """

    base_fee_change_denominator: int = betterproto.uint32_field(2)
    """
    base fee change denominator bounds the amount the base fee can change
     between blocks.
    """

    elasticity_multiplier: int = betterproto.uint32_field(3)
    """
    elasticity multiplier bounds the maximum gas limit an EIP-1559 block may
     have.
    """

    enable_height: int = betterproto.int64_field(5)
    """height at which the base fee calculation is enabled."""

    base_fee: str = betterproto.string_field(6)
    """base fee for EIP-1559 blocks."""

    min_gas_price: str = betterproto.string_field(7)
    """
    min_gas_price defines the minimum gas price value for cosmos and eth transactions
    """

    min_gas_multiplier: str = betterproto.string_field(8)
    """
    min gas denominator bounds the minimum gasUsed to be charged
     to senders based on GasLimit
    """


@dataclass(eq=False, repr=False)
class QueryParamsRequest(betterproto.Message):
    """
    QueryParamsRequest defines the request type for querying x/evm parameters.
    """

    pass


@dataclass(eq=False, repr=False)
class QueryParamsResponse(betterproto.Message):
    """
    QueryParamsResponse defines the response type for querying x/evm parameters.
    """

    params: "Params" = betterproto.message_field(1)
    """params define the evm module parameters."""


@dataclass(eq=False, repr=False)
class QueryBaseFeeRequest(betterproto.Message):
    """
    QueryBaseFeeRequest defines the request type for querying the EIP1559 base
     fee.
    """

    pass


@dataclass(eq=False, repr=False)
class QueryBaseFeeResponse(betterproto.Message):
    """BaseFeeResponse returns the EIP1559 base fee."""

    base_fee: str = betterproto.string_field(1)


@dataclass(eq=False, repr=False)
class QueryBlockGasRequest(betterproto.Message):
    """
    QueryBlockGasRequest defines the request type for querying the EIP1559 base
     fee.
    """

    pass


@dataclass(eq=False, repr=False)
class QueryBlockGasResponse(betterproto.Message):
    """QueryBlockGasResponse returns block gas used for a given height."""

    gas: int = betterproto.int64_field(1)


@dataclass(eq=False, repr=False)
class GenesisState(betterproto.Message):
    """GenesisState defines the feemarket module's genesis state."""

    params: "Params" = betterproto.message_field(1)
    """params defines all the paramaters of the module."""

    block_gas: int = betterproto.uint64_field(3)
    """
    block gas is the amount of gas wanted on the last block before the upgrade.
     Zero by default.
    """


class QueryStub(betterproto.ServiceStub):
    async def params(
        self,
        query_params_request: "QueryParamsRequest",
        *,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None
    ) -> "QueryParamsResponse":
        return await self._unary_unary(
            "/ethermint.feemarket.v1.Query/Params",
            query_params_request,
            QueryParamsResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    async def base_fee(
        self,
        query_base_fee_request: "QueryBaseFeeRequest",
        *,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None
    ) -> "QueryBaseFeeResponse":
        return await self._unary_unary(
            "/ethermint.feemarket.v1.Query/BaseFee",
            query_base_fee_request,
            QueryBaseFeeResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    async def block_gas(
        self,
        query_block_gas_request: "QueryBlockGasRequest",
        *,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None
    ) -> "QueryBlockGasResponse":
        return await self._unary_unary(
            "/ethermint.feemarket.v1.Query/BlockGas",
            query_block_gas_request,
            QueryBlockGasResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )


class QueryBase(ServiceBase):
    async def params(
        self, query_params_request: "QueryParamsRequest"
    ) -> "QueryParamsResponse":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def base_fee(
        self, query_base_fee_request: "QueryBaseFeeRequest"
    ) -> "QueryBaseFeeResponse":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def block_gas(
        self, query_block_gas_request: "QueryBlockGasRequest"
    ) -> "QueryBlockGasResponse":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def __rpc_params(
        self, stream: "grpclib.server.Stream[QueryParamsRequest, QueryParamsResponse]"
    ) -> None:
        request = await stream.recv_message()
        response = await self.params(request)
        await stream.send_message(response)

    async def __rpc_base_fee(
        self, stream: "grpclib.server.Stream[QueryBaseFeeRequest, QueryBaseFeeResponse]"
    ) -> None:
        request = await stream.recv_message()
        response = await self.base_fee(request)
        await stream.send_message(response)

    async def __rpc_block_gas(
        self,
        stream: "grpclib.server.Stream[QueryBlockGasRequest, QueryBlockGasResponse]",
    ) -> None:
        request = await stream.recv_message()
        response = await self.block_gas(request)
        await stream.send_message(response)

    def __mapping__(self) -> Dict[str, grpclib.const.Handler]:
        return {
            "/ethermint.feemarket.v1.Query/Params": grpclib.const.Handler(
                self.__rpc_params,
                grpclib.const.Cardinality.UNARY_UNARY,
                QueryParamsRequest,
                QueryParamsResponse,
            ),
            "/ethermint.feemarket.v1.Query/BaseFee": grpclib.const.Handler(
                self.__rpc_base_fee,
                grpclib.const.Cardinality.UNARY_UNARY,
                QueryBaseFeeRequest,
                QueryBaseFeeResponse,
            ),
            "/ethermint.feemarket.v1.Query/BlockGas": grpclib.const.Handler(
                self.__rpc_block_gas,
                grpclib.const.Cardinality.UNARY_UNARY,
                QueryBlockGasRequest,
                QueryBlockGasResponse,
            ),
        }



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# sources: ibc/lightclients/tendermint/v1/tendermint.proto
# plugin: python-betterproto
# This file has been @generated

from dataclasses import dataclass
from datetime import (
    datetime,
    timedelta,
)
from typing import List

import betterproto

from ..... import ics23 as ____ics23__
from .....tendermint import types as ____tendermint_types__
from ....core.client import v1 as ___core_client_v1__
from ....core.commitment import v1 as ___core_commitment_v1__


@dataclass(eq=False, repr=False)
class ClientState(betterproto.Message):
    """
    ClientState from Tendermint tracks the current validator set, latest height,
     and a possible frozen height.
    """

    chain_id: str = betterproto.string_field(1)
    trust_level: "Fraction" = betterproto.message_field(2)
    trusting_period: timedelta = betterproto.message_field(3)
    """
    duration of the period since the LastestTimestamp during which the
     submitted headers are valid for upgrade
    """

    unbonding_period: timedelta = betterproto.message_field(4)
    """duration of the staking unbonding period"""

    max_clock_drift: timedelta = betterproto.message_field(5)
    """
    defines how much new (untrusted) header's Time can drift into the future.
    """

    frozen_height: "___core_client_v1__.Height" = betterproto.message_field(6)
    """Block height when the client was frozen due to a misbehaviour"""

    latest_height: "___core_client_v1__.Height" = betterproto.message_field(7)
    """Latest height the client was updated to"""

    proof_specs: List["____ics23__.ProofSpec"] = betterproto.message_field(8)
    """Proof specifications used in verifying counterparty state"""

    upgrade_path: List[str] = betterproto.string_field(9)
    """
    Path at which next upgraded client will be committed.
     Each element corresponds to the key for a single CommitmentProof in the
     chained proof. NOTE: ClientState must stored under
     `{upgradePath}/{upgradeHeight}/clientState` ConsensusState must be stored
     under `{upgradepath}/{upgradeHeight}/consensusState` For SDK chains using
     the default upgrade module, upgrade_path should be []string{"upgrade",
     "upgradedIBCState"}`
    """

    allow_update_after_expiry: bool = betterproto.bool_field(10)
    """
    This flag, when set to true, will allow governance to recover a client
     which has expired
    """

    allow_update_after_misbehaviour: bool = betterproto.bool_field(11)
    """
    This flag, when set to true, will allow governance to unfreeze a client
     whose chain has experienced a misbehaviour event
    """


@dataclass(eq=False, repr=False)
class ConsensusState(betterproto.Message):
    """ConsensusState defines the consensus state from Tendermint."""

    timestamp: datetime = betterproto.message_field(1)
    """
    timestamp that corresponds to the block height in which the ConsensusState
     was stored.
    """

    root: "___core_commitment_v1__.MerkleRoot" = betterproto.message_field(2)
    """commitment root (i.e app hash)"""

    next_validators_hash: bytes = betterproto.bytes_field(3)


@dataclass(eq=False, repr=False)
class Misbehaviour(betterproto.Message):
    """
    Misbehaviour is a wrapper over two conflicting Headers
     that implements Misbehaviour interface expected by ICS-02
    """

    client_id: str = betterproto.string_field(1)
    header_1: "Header" = betterproto.message_field(2)
    header_2: "Header" = betterproto.message_field(3)


@dataclass(eq=False, repr=False)
class Header(betterproto.Message):
    """
    Header defines the Tendermint client consensus Header.
     It encapsulates all the information necessary to update from a trusted
     Tendermint ConsensusState. The inclusion of TrustedHeight and
     TrustedValidators allows this update to process correctly, so long as the
     ConsensusState for the TrustedHeight exists, this removes race conditions
     among relayers The SignedHeader and ValidatorSet are the new untrusted update
     fields for the client. The TrustedHeight is the height of a stored
     ConsensusState on the client that will be used to verify the new untrusted
     header. The Trusted ConsensusState must be within the unbonding period of
     current time in order to correctly verify, and the TrustedValidators must
     hash to TrustedConsensusState.NextValidatorsHash since that is the last
     trusted validator set at the TrustedHeight.
    """

    signed_header: "____tendermint_types__.SignedHeader" = betterproto.message_field(1)
    validator_set: "____tendermint_types__.ValidatorSet" = betterproto.message_field(2)
    trusted_height: "___core_client_v1__.Height" = betterproto.message_field(3)
    trusted_validators: "____tendermint_types__.ValidatorSet" = (
        betterproto.message_field(4)
    )


@dataclass(eq=False, repr=False)
class Fraction(betterproto.Message):
    """
    Fraction defines the protobuf message type for tmmath.Fraction that only
     supports positive values.
    """

    numerator: int = betterproto.uint64_field(1)
    denominator: int = betterproto.uint64_field(2)



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# sources: ibc/applications/interchain_accounts/v1/account.proto, ibc/applications/interchain_accounts/v1/genesis.proto, ibc/applications/interchain_accounts/v1/metadata.proto, ibc/applications/interchain_accounts/v1/packet.proto
# plugin: python-betterproto
# This file has been @generated

from dataclasses import dataclass
from typing import List

import betterproto
import betterproto.lib.google.protobuf as betterproto_lib_google_protobuf

from .....cosmos.auth import v1beta1 as ____cosmos_auth_v1_beta1__
from ..controller import v1 as _controller_v1__
from ..host import v1 as _host_v1__


class Type(betterproto.Enum):
    """
    Type defines a classification of message issued from a controller chain to its associated interchain accounts
     host
    """

    UNSPECIFIED = 0
    """Default zero value enumeration"""

    EXECUTE_TX = 1
    """Execute a transaction on an interchain accounts host chain"""


@dataclass(eq=False, repr=False)
class InterchainAccount(betterproto.Message):
    """
    An InterchainAccount is defined as a BaseAccount & the address of the account owner on the controller chain
    """

    base_account: "____cosmos_auth_v1_beta1__.BaseAccount" = betterproto.message_field(
        1
    )
    account_owner: str = betterproto.string_field(2)


@dataclass(eq=False, repr=False)
class InterchainAccountPacketData(betterproto.Message):
    """
    InterchainAccountPacketData is comprised of a raw transaction, type of transaction and optional memo field.
    """

    type: "Type" = betterproto.enum_field(1)
    data: bytes = betterproto.bytes_field(2)
    memo: str = betterproto.string_field(3)


@dataclass(eq=False, repr=False)
class CosmosTx(betterproto.Message):
    """
    CosmosTx contains a list of sdk.Msg's. It should be used when sending transactions to an SDK host chain.
    """

    messages: List["betterproto_lib_google_protobuf.Any"] = betterproto.message_field(1)


@dataclass(eq=False, repr=False)
class Metadata(betterproto.Message):
    """
    Metadata defines a set of protocol specific data encoded into the ICS27 channel version bytestring
     See ICS004: https://github.com/cosmos/ibc/tree/master/spec/core/ics-004-channel-and-packet-semantics#Versioning
    """

    version: str = betterproto.string_field(1)
    """version defines the ICS27 protocol version"""

    controller_connection_id: str = betterproto.string_field(2)
    """
    controller_connection_id is the connection identifier associated with the controller chain
    """

    host_connection_id: str = betterproto.string_field(3)
    """
    host_connection_id is the connection identifier associated with the host chain
    """

    address: str = betterproto.string_field(4)
    """
    address defines the interchain account address to be fulfilled upon the OnChanOpenTry handshake step
     NOTE: the address field is empty on the OnChanOpenInit handshake step
    """

    encoding: str = betterproto.string_field(5)
    """encoding defines the supported codec format"""

    tx_type: str = betterproto.string_field(6)
    """
    tx_type defines the type of transactions the interchain account can execute
    """


@dataclass(eq=False, repr=False)
class GenesisState(betterproto.Message):
    """GenesisState defines the interchain accounts genesis state"""

    controller_genesis_state: "ControllerGenesisState" = betterproto.message_field(1)
    host_genesis_state: "HostGenesisState" = betterproto.message_field(2)


@dataclass(eq=False, repr=False)
class ControllerGenesisState(betterproto.Message):
    """
    ControllerGenesisState defines the interchain accounts controller genesis state
    """

    active_channels: List["ActiveChannel"] = betterproto.message_field(1)
    interchain_accounts: List[
        "RegisteredInterchainAccount"
    ] = betterproto.message_field(2)
    ports: List[str] = betterproto.string_field(3)
    params: "_controller_v1__.Params" = betterproto.message_field(4)


@dataclass(eq=False, repr=False)
class HostGenesisState(betterproto.Message):
    """HostGenesisState defines the interchain accounts host genesis state"""

    active_channels: List["ActiveChannel"] = betterproto.message_field(1)
    interchain_accounts: List[
        "RegisteredInterchainAccount"
    ] = betterproto.message_field(2)
    port: str = betterproto.string_field(3)
    params: "_host_v1__.Params" = betterproto.message_field(4)


@dataclass(eq=False, repr=False)
class ActiveChannel(betterproto.Message):
    """
    ActiveChannel contains a connection ID, port ID and associated active channel ID
    """

    connection_id: str = betterproto.string_field(1)
    port_id: str = betterproto.string_field(2)
    channel_id: str = betterproto.string_field(3)


@dataclass(eq=False, repr=False)
class RegisteredInterchainAccount(betterproto.Message):
    """
    RegisteredInterchainAccount contains a connection ID, port ID and associated interchain account address
    """

    connection_id: str = betterproto.string_field(1)
    port_id: str = betterproto.string_field(2)
    account_address: str = betterproto.string_field(3)



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# sources: ibc/applications/transfer/v2/packet.proto
# plugin: python-betterproto
# This file has been @generated

from dataclasses import dataclass

import betterproto


@dataclass(eq=False, repr=False)
class FungibleTokenPacketData(betterproto.Message):
    """
    FungibleTokenPacketData defines a struct for the packet payload
     See FungibleTokenPacketData spec:
     https://github.com/cosmos/ibc/tree/master/spec/app/ics-020-fungible-token-transfer#data-structures
    """

    denom: str = betterproto.string_field(1)
    """the token denomination to be transferred"""

    amount: str = betterproto.string_field(2)
    """the token amount to be transferred"""

    sender: str = betterproto.string_field(3)
    """the sender address"""

    receiver: str = betterproto.string_field(4)
    """the recipient address on the destination chain"""



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# sources: ethermint/evm/v1/evm.proto, ethermint/evm/v1/genesis.proto, ethermint/evm/v1/query.proto, ethermint/evm/v1/tx.proto
# plugin: python-betterproto
# This file has been @generated

from dataclasses import dataclass
from datetime import datetime
from typing import (
    TYPE_CHECKING,
    Dict,
    List,
    Optional,
)

import betterproto
import betterproto.lib.google.protobuf as betterproto_lib_google_protobuf
import grpclib
from betterproto.grpc.grpclib_server import ServiceBase

from ....cosmos.base.query import v1beta1 as ___cosmos_base_query_v1_beta1__


if TYPE_CHECKING:
    import grpclib.server
    from betterproto.grpc.grpclib_client import MetadataLike
    from grpclib.metadata import Deadline


@dataclass(eq=False, repr=False)
class Params(betterproto.Message):
    """Params defines the EVM module parameters"""

    evm_denom: str = betterproto.string_field(1)
    """
    evm denom represents the token denomination used to run the EVM state
     transitions.
    """

    enable_create: bool = betterproto.bool_field(2)
    """
    enable create toggles state transitions that use the vm.Create function
    """

    enable_call: bool = betterproto.bool_field(3)
    """enable call toggles state transitions that use the vm.Call function"""

    extra_eips: List[int] = betterproto.int64_field(4)
    """extra eips defines the additional EIPs for the vm.Config"""

    chain_config: "ChainConfig" = betterproto.message_field(5)
    """chain config defines the EVM chain configuration parameters"""

    allow_unprotected_txs: bool = betterproto.bool_field(6)
    """
    Allow unprotected transactions defines if replay-protected (i.e non EIP155 
     signed) transactions can be executed on the state machine.
    """


@dataclass(eq=False, repr=False)
class ChainConfig(betterproto.Message):
    """
    ChainConfig defines the Ethereum ChainConfig parameters using *sdk.Int values
     instead of *big.Int.
    """

    homestead_block: str = betterproto.string_field(1)
    """Homestead switch block (nil no fork, 0 = already homestead)"""

    dao_fork_block: str = betterproto.string_field(2)
    """TheDAO hard-fork switch block (nil no fork)"""

    dao_fork_support: bool = betterproto.bool_field(3)
    """Whether the nodes supports or opposes the DAO hard-fork"""

    eip150_block: str = betterproto.string_field(4)
    """
    EIP150 implements the Gas price changes
     (https://github.com/ethereum/EIPs/issues/150) EIP150 HF block (nil no fork)
    """

    eip150_hash: str = betterproto.string_field(5)
    """
    EIP150 HF hash (needed for header only clients as only gas pricing changed)
    """

    eip155_block: str = betterproto.string_field(6)
    """EIP155Block HF block"""

    eip158_block: str = betterproto.string_field(7)
    """EIP158 HF block"""

    byzantium_block: str = betterproto.string_field(8)
    """Byzantium switch block (nil no fork, 0 = already on byzantium)"""

    constantinople_block: str = betterproto.string_field(9)
    """Constantinople switch block (nil no fork, 0 = already activated)"""

    petersburg_block: str = betterproto.string_field(10)
    """Petersburg switch block (nil same as Constantinople)"""

    istanbul_block: str = betterproto.string_field(11)
    """Istanbul switch block (nil no fork, 0 = already on istanbul)"""

    muir_glacier_block: str = betterproto.string_field(12)
    """
    Eip-2384 (bomb delay) switch block (nil no fork, 0 = already activated)
    """

    berlin_block: str = betterproto.string_field(13)
    """Berlin switch block (nil = no fork, 0 = already on berlin)"""

    london_block: str = betterproto.string_field(17)
    """London switch block (nil = no fork, 0 = already on london)"""

    arrow_glacier_block: str = betterproto.string_field(18)
    """
    Eip-4345 (bomb delay) switch block (nil = no fork, 0 = already activated)
    """

    merge_fork_block: str = betterproto.string_field(19)
    """
    EIP-3675 (TheMerge) switch block (nil = no fork, 0 = already in merge proceedings)
    """


@dataclass(eq=False, repr=False)
class State(betterproto.Message):
    """State represents a single Storage key value pair item."""

    key: str = betterproto.string_field(1)
    value: str = betterproto.string_field(2)


@dataclass(eq=False, repr=False)
class TransactionLogs(betterproto.Message):
    """
    TransactionLogs define the logs generated from a transaction execution
     with a given hash. It it used for import/export data as transactions are not
     persisted on blockchain state after an upgrade.
    """

    hash: str = betterproto.string_field(1)
    logs: List["Log"] = betterproto.message_field(2)


@dataclass(eq=False, repr=False)
class Log(betterproto.Message):
    """
    Log represents an protobuf compatible Ethereum Log that defines a contract
     log event. These events are generated by the LOG opcode and stored/indexed by
     the node.
    """

    address: str = betterproto.string_field(1)
    """address of the contract that generated the event"""

    topics: List[str] = betterproto.string_field(2)
    """list of topics provided by the contract."""

    data: bytes = betterproto.bytes_field(3)
    """supplied by the contract, usually ABI-encoded"""

    block_number: int = betterproto.uint64_field(4)
    """block in which the transaction was included"""

    tx_hash: str = betterproto.string_field(5)
    """hash of the transaction"""

    tx_index: int = betterproto.uint64_field(6)
    """index of the transaction in the block"""

    block_hash: str = betterproto.string_field(7)
    """hash of the block in which the transaction was included"""

    index: int = betterproto.uint64_field(8)
    """index of the log in the block"""

    removed: bool = betterproto.bool_field(9)
    """
    The Removed field is true if this log was reverted due to a chain
     reorganisation. You must pay attention to this field if you receive logs
     through a filter query.
    """


@dataclass(eq=False, repr=False)
class TxResult(betterproto.Message):
    """TxResult stores results of Tx execution."""

    contract_address: str = betterproto.string_field(1)
    """
    contract_address contains the ethereum address of the created contract (if
     any). If the state transition is an evm.Call, the contract address will be
     empty.
    """

    bloom: bytes = betterproto.bytes_field(2)
    """bloom represents the bloom filter bytes"""

    tx_logs: "TransactionLogs" = betterproto.message_field(3)
    """
    tx_logs contains the transaction hash and the proto-compatible ethereum
     logs.
    """

    ret: bytes = betterproto.bytes_field(4)
    """ret defines the bytes from the execution."""

    reverted: bool = betterproto.bool_field(5)
    """reverted flag is set to true when the call has been reverted"""

    gas_used: int = betterproto.uint64_field(6)
    """gas_used notes the amount of gas consumed while execution"""


@dataclass(eq=False, repr=False)
class AccessTuple(betterproto.Message):
    """AccessTuple is the element type of an access list."""

    address: str = betterproto.string_field(1)
    """hex formatted ethereum address"""

    storage_keys: List[str] = betterproto.string_field(2)
    """hex formatted hashes of the storage keys"""


@dataclass(eq=False, repr=False)
class TraceConfig(betterproto.Message):
    """TraceConfig holds extra parameters to trace functions."""

    tracer: str = betterproto.string_field(1)
    """custom javascript tracer"""

    timeout: str = betterproto.string_field(2)
    """
    overrides the default timeout of 5 seconds for JavaScript-based tracing
     calls
    """

    reexec: int = betterproto.uint64_field(3)
    """number of blocks the tracer is willing to go back"""

    disable_stack: bool = betterproto.bool_field(5)
    """disable stack capture"""

    disable_storage: bool = betterproto.bool_field(6)
    """disable storage capture"""

    debug: bool = betterproto.bool_field(8)
    """print output during capture end"""

    limit: int = betterproto.int32_field(9)
    """maximum length of output, but zero means unlimited"""

    overrides: "ChainConfig" = betterproto.message_field(10)
    """
    Chain overrides, can be used to execute a trace using future fork rules
    """

    enable_memory: bool = betterproto.bool_field(11)
    """enable memory capture"""

    enable_return_data: bool = betterproto.bool_field(12)
    """enable return data capture"""


@dataclass(eq=False, repr=False)
class MsgEthereumTx(betterproto.Message):
    """
    MsgEthereumTx encapsulates an Ethereum transaction as an SDK message.
    """

    data: "betterproto_lib_google_protobuf.Any" = betterproto.message_field(1)
    """inner transaction data"""

    size: float = betterproto.double_field(2)
    """encoded storage size of the transaction"""

    hash: str = betterproto.string_field(3)
    """transaction hash in hex format"""

    from_: str = betterproto.string_field(4)
    """
    ethereum signer address in hex format. This address value is checked
     against the address derived from the signature (V, R, S) using the
     secp256k1 elliptic curve
    """


@dataclass(eq=False, repr=False)
class LegacyTx(betterproto.Message):
    """
    LegacyTx is the transaction data of regular Ethereum transactions.
     NOTE: All non-protected transactions (i.e non EIP155 signed) will fail if the
     AllowUnprotectedTxs parameter is disabled.
    """

    nonce: int = betterproto.uint64_field(1)
    """nonce corresponds to the account nonce (transaction sequence)."""

    gas_price: str = betterproto.string_field(2)
    """gas price defines the value for each gas unit"""

    gas: int = betterproto.uint64_field(3)
    """gas defines the gas limit defined for the transaction."""

    to: str = betterproto.string_field(4)
    """hex formatted address of the recipient"""

    value: str = betterproto.string_field(5)
    """value defines the unsigned integer value of the transaction amount."""

    data: bytes = betterproto.bytes_field(6)
    """input defines the data payload bytes of the transaction."""

    v: bytes = betterproto.bytes_field(7)
    """v defines the signature value"""

    r: bytes = betterproto.bytes_field(8)
    """r defines the signature value"""

    s: bytes = betterproto.bytes_field(9)
    """s define the signature value"""


@dataclass(eq=False, repr=False)
class AccessListTx(betterproto.Message):
    """AccessListTx is the data of EIP-2930 access list transactions."""

    chain_id: str = betterproto.string_field(1)
    """destination EVM chain ID"""

    nonce: int = betterproto.uint64_field(2)
    """nonce corresponds to the account nonce (transaction sequence)."""

    gas_price: str = betterproto.string_field(3)
    """gas price defines the value for each gas unit"""

    gas: int = betterproto.uint64_field(4)
    """gas defines the gas limit defined for the transaction."""

    to: str = betterproto.string_field(5)
    """hex formatted address of the recipient"""

    value: str = betterproto.string_field(6)
    """value defines the unsigned integer value of the transaction amount."""

    data: bytes = betterproto.bytes_field(7)
    """input defines the data payload bytes of the transaction."""

    accesses: List["AccessTuple"] = betterproto.message_field(8)
    v: bytes = betterproto.bytes_field(9)
    """v defines the signature value"""

    r: bytes = betterproto.bytes_field(10)
    """r defines the signature value"""

    s: bytes = betterproto.bytes_field(11)
    """s define the signature value"""


@dataclass(eq=False, repr=False)
class DynamicFeeTx(betterproto.Message):
    """DynamicFeeTx is the data of EIP-1559 dinamic fee transactions."""

    chain_id: str = betterproto.string_field(1)
    """destination EVM chain ID"""

    nonce: int = betterproto.uint64_field(2)
    """nonce corresponds to the account nonce (transaction sequence)."""

    gas_tip_cap: str = betterproto.string_field(3)
    """gas tip cap defines the max value for the gas tip"""

    gas_fee_cap: str = betterproto.string_field(4)
    """gas fee cap defines the max value for the gas fee"""

    gas: int = betterproto.uint64_field(5)
    """gas defines the gas limit defined for the transaction."""

    to: str = betterproto.string_field(6)
    """hex formatted address of the recipient"""

    value: str = betterproto.string_field(7)
    """value defines the the transaction amount."""

    data: bytes = betterproto.bytes_field(8)
    """input defines the data payload bytes of the transaction."""

    accesses: List["AccessTuple"] = betterproto.message_field(9)
    v: bytes = betterproto.bytes_field(10)
    """v defines the signature value"""

    r: bytes = betterproto.bytes_field(11)
    """r defines the signature value"""

    s: bytes = betterproto.bytes_field(12)
    """s define the signature value"""


@dataclass(eq=False, repr=False)
class ExtensionOptionsEthereumTx(betterproto.Message):
    pass


@dataclass(eq=False, repr=False)
class MsgEthereumTxResponse(betterproto.Message):
    """MsgEthereumTxResponse defines the Msg/EthereumTx response type."""

    hash: str = betterproto.string_field(1)
    """
    ethereum transaction hash in hex format. This hash differs from the
     Tendermint sha256 hash of the transaction bytes. See
     https://github.com/tendermint/tendermint/issues/6539 for reference
    """

    logs: List["Log"] = betterproto.message_field(2)
    """
    logs contains the transaction hash and the proto-compatible ethereum
     logs.
    """

    ret: bytes = betterproto.bytes_field(3)
    """
    returned data from evm function (result or data supplied with revert
     opcode)
    """

    vm_error: str = betterproto.string_field(4)
    """vm error is the error returned by vm execution"""

    gas_used: int = betterproto.uint64_field(5)
    """gas consumed by the transaction"""


@dataclass(eq=False, repr=False)
class QueryAccountRequest(betterproto.Message):
    """
    QueryAccountRequest is the request type for the Query/Account RPC method.
    """

    address: str = betterproto.string_field(1)
    """address is the ethereum hex address to query the account for."""


@dataclass(eq=False, repr=False)
class QueryAccountResponse(betterproto.Message):
    """
    QueryAccountResponse is the response type for the Query/Account RPC method.
    """

    balance: str = betterproto.string_field(1)
    """balance is the balance of the EVM denomination."""

    code_hash: str = betterproto.string_field(2)
    """code hash is the hex-formatted code bytes from the EOA."""

    nonce: int = betterproto.uint64_field(3)
    """nonce is the account's sequence number."""


@dataclass(eq=False, repr=False)
class QueryCosmosAccountRequest(betterproto.Message):
    """
    QueryCosmosAccountRequest is the request type for the Query/CosmosAccount RPC
     method.
    """

    address: str = betterproto.string_field(1)
    """address is the ethereum hex address to query the account for."""


@dataclass(eq=False, repr=False)
class QueryCosmosAccountResponse(betterproto.Message):
    """
    QueryCosmosAccountResponse is the response type for the Query/CosmosAccount
     RPC method.
    """

    cosmos_address: str = betterproto.string_field(1)
    """cosmos_address is the cosmos address of the account."""

    sequence: int = betterproto.uint64_field(2)
    """sequence is the account's sequence number."""

    account_number: int = betterproto.uint64_field(3)
    """account_number is the account numbert"""


@dataclass(eq=False, repr=False)
class QueryValidatorAccountRequest(betterproto.Message):
    """
    QueryValidatorAccountRequest is the request type for the
     Query/ValidatorAccount RPC method.
    """

    cons_address: str = betterproto.string_field(1)
    """cons_address is the validator cons address to query the account for."""


@dataclass(eq=False, repr=False)
class QueryValidatorAccountResponse(betterproto.Message):
    """
    QueryValidatorAccountResponse is the response type for the
     Query/ValidatorAccount RPC method.
    """

    account_address: str = betterproto.string_field(1)
    """
    account_address is the cosmos address of the account in bech32 format.
    """

    sequence: int = betterproto.uint64_field(2)
    """sequence is the account's sequence number."""

    account_number: int = betterproto.uint64_field(3)
    """account_number is the account number"""


@dataclass(eq=False, repr=False)
class QueryBalanceRequest(betterproto.Message):
    """
    QueryBalanceRequest is the request type for the Query/Balance RPC method.
    """

    address: str = betterproto.string_field(1)
    """address is the ethereum hex address to query the balance for."""


@dataclass(eq=False, repr=False)
class QueryBalanceResponse(betterproto.Message):
    """
    QueryBalanceResponse is the response type for the Query/Balance RPC method.
    """

    balance: str = betterproto.string_field(1)
    """balance is the balance of the EVM denomination."""


@dataclass(eq=False, repr=False)
class QueryStorageRequest(betterproto.Message):
    """
    QueryStorageRequest is the request type for the Query/Storage RPC method.
    """

    address: str = betterproto.string_field(1)
    """
    / address is the ethereum hex address to query the storage state for.
    """

    key: str = betterproto.string_field(2)
    """key defines the key of the storage state"""


@dataclass(eq=False, repr=False)
class QueryStorageResponse(betterproto.Message):
    """
    QueryStorageResponse is the response type for the Query/Storage RPC
     method.
    """

    value: str = betterproto.string_field(1)
    """
    key defines the storage state value hash associated with the given key.
    """


@dataclass(eq=False, repr=False)
class QueryCodeRequest(betterproto.Message):
    """QueryCodeRequest is the request type for the Query/Code RPC method."""

    address: str = betterproto.string_field(1)
    """address is the ethereum hex address to query the code for."""


@dataclass(eq=False, repr=False)
class QueryCodeResponse(betterproto.Message):
    """
    QueryCodeResponse is the response type for the Query/Code RPC
     method.
    """

    code: bytes = betterproto.bytes_field(1)
    """code represents the code bytes from an ethereum address."""


@dataclass(eq=False, repr=False)
class QueryTxLogsRequest(betterproto.Message):
    """
    QueryTxLogsRequest is the request type for the Query/TxLogs RPC method.
    """

    hash: str = betterproto.string_field(1)
    """hash is the ethereum transaction hex hash to query the logs for."""

    pagination: "___cosmos_base_query_v1_beta1__.PageRequest" = (
        betterproto.message_field(2)
    )
    """pagination defines an optional pagination for the request."""


@dataclass(eq=False, repr=False)
class QueryTxLogsResponse(betterproto.Message):
    """QueryTxLogs is the response type for the Query/TxLogs RPC method."""

    logs: List["Log"] = betterproto.message_field(1)
    """
    logs represents the ethereum logs generated from the given transaction.
    """

    pagination: "___cosmos_base_query_v1_beta1__.PageResponse" = (
        betterproto.message_field(2)
    )
    """pagination defines the pagination in the response."""


@dataclass(eq=False, repr=False)
class QueryParamsRequest(betterproto.Message):
    """
    QueryParamsRequest defines the request type for querying x/evm parameters.
    """

    pass


@dataclass(eq=False, repr=False)
class QueryParamsResponse(betterproto.Message):
    """
    QueryParamsResponse defines the response type for querying x/evm parameters.
    """

    params: "Params" = betterproto.message_field(1)
    """params define the evm module parameters."""


@dataclass(eq=False, repr=False)
class EthCallRequest(betterproto.Message):
    """EthCallRequest defines EthCall request"""

    args: bytes = betterproto.bytes_field(1)
    """same json format as the json rpc api."""

    gas_cap: int = betterproto.uint64_field(2)
    """the default gas cap to be used"""


@dataclass(eq=False, repr=False)
class EstimateGasResponse(betterproto.Message):
    """EstimateGasResponse defines EstimateGas response"""

    gas: int = betterproto.uint64_field(1)
    """the estimated gas"""


@dataclass(eq=False, repr=False)
class QueryTraceTxRequest(betterproto.Message):
    """QueryTraceTxRequest defines TraceTx request"""

    msg: "MsgEthereumTx" = betterproto.message_field(1)
    """msgEthereumTx for the requested transaction"""

    trace_config: "TraceConfig" = betterproto.message_field(3)
    """TraceConfig holds extra parameters to trace functions."""

    predecessors: List["MsgEthereumTx"] = betterproto.message_field(4)
    """
    the predecessor transactions included in the same block
     need to be replayed first to get correct context for tracing.
    """

    block_number: int = betterproto.int64_field(5)
    """block number of requested transaction"""

    block_hash: str = betterproto.string_field(6)
    """block hex hash of requested transaction"""

    block_time: datetime = betterproto.message_field(7)
    """block time of requested transaction"""


@dataclass(eq=False, repr=False)
class QueryTraceTxResponse(betterproto.Message):
    """QueryTraceTxResponse defines TraceTx response"""

    data: bytes = betterproto.bytes_field(1)
    """response serialized in bytes"""


@dataclass(eq=False, repr=False)
class QueryTraceBlockRequest(betterproto.Message):
    """QueryTraceBlockRequest defines TraceTx request"""

    txs: List["MsgEthereumTx"] = betterproto.message_field(1)
    """txs messages in the block"""

    trace_config: "TraceConfig" = betterproto.message_field(3)
    """TraceConfig holds extra parameters to trace functions."""

    block_number: int = betterproto.int64_field(5)
    """block number"""

    block_hash: str = betterproto.string_field(6)
    """block hex hash"""

    block_time: datetime = betterproto.message_field(7)
    """block time"""


@dataclass(eq=False, repr=False)
class QueryTraceBlockResponse(betterproto.Message):
    """QueryTraceBlockResponse defines TraceBlock response"""

    data: bytes = betterproto.bytes_field(1)


@dataclass(eq=False, repr=False)
class QueryBaseFeeRequest(betterproto.Message):
    """
    QueryBaseFeeRequest defines the request type for querying the EIP1559 base
     fee.
    """

    pass


@dataclass(eq=False, repr=False)
class QueryBaseFeeResponse(betterproto.Message):
    """BaseFeeResponse returns the EIP1559 base fee."""

    base_fee: str = betterproto.string_field(1)


@dataclass(eq=False, repr=False)
class GenesisState(betterproto.Message):
    """GenesisState defines the evm module's genesis state."""

    accounts: List["GenesisAccount"] = betterproto.message_field(1)
    """accounts is an array containing the ethereum genesis accounts."""

    params: "Params" = betterproto.message_field(2)
    """params defines all the parameters of the module."""


@dataclass(eq=False, repr=False)
class GenesisAccount(betterproto.Message):
    """
    GenesisAccount defines an account to be initialized in the genesis state.
     Its main difference between with Geth's GenesisAccount is that it uses a
     custom storage type and that it doesn't contain the private key field.
    """

    address: str = betterproto.string_field(1)
    """address defines an ethereum hex formated address of an account"""

    code: str = betterproto.string_field(2)
    """code defines the hex bytes of the account code."""

    storage: List["State"] = betterproto.message_field(3)
    """storage defines the set of state key values for the account."""


class MsgStub(betterproto.ServiceStub):
    async def ethereum_tx(
        self,
        msg_ethereum_tx: "MsgEthereumTx",
        *,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None
    ) -> "MsgEthereumTxResponse":
        return await self._unary_unary(
            "/ethermint.evm.v1.Msg/EthereumTx",
            msg_ethereum_tx,
            MsgEthereumTxResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )


class QueryStub(betterproto.ServiceStub):
    async def account(
        self,
        query_account_request: "QueryAccountRequest",
        *,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None
    ) -> "QueryAccountResponse":
        return await self._unary_unary(
            "/ethermint.evm.v1.Query/Account",
            query_account_request,
            QueryAccountResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    async def cosmos_account(
        self,
        query_cosmos_account_request: "QueryCosmosAccountRequest",
        *,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None
    ) -> "QueryCosmosAccountResponse":
        return await self._unary_unary(
            "/ethermint.evm.v1.Query/CosmosAccount",
            query_cosmos_account_request,
            QueryCosmosAccountResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    async def validator_account(
        self,
        query_validator_account_request: "QueryValidatorAccountRequest",
        *,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None
    ) -> "QueryValidatorAccountResponse":
        return await self._unary_unary(
            "/ethermint.evm.v1.Query/ValidatorAccount",
            query_validator_account_request,
            QueryValidatorAccountResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    async def balance(
        self,
        query_balance_request: "QueryBalanceRequest",
        *,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None
    ) -> "QueryBalanceResponse":
        return await self._unary_unary(
            "/ethermint.evm.v1.Query/Balance",
            query_balance_request,
            QueryBalanceResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    async def storage(
        self,
        query_storage_request: "QueryStorageRequest",
        *,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None
    ) -> "QueryStorageResponse":
        return await self._unary_unary(
            "/ethermint.evm.v1.Query/Storage",
            query_storage_request,
            QueryStorageResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    async def code(
        self,
        query_code_request: "QueryCodeRequest",
        *,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None
    ) -> "QueryCodeResponse":
        return await self._unary_unary(
            "/ethermint.evm.v1.Query/Code",
            query_code_request,
            QueryCodeResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    async def params(
        self,
        query_params_request: "QueryParamsRequest",
        *,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None
    ) -> "QueryParamsResponse":
        return await self._unary_unary(
            "/ethermint.evm.v1.Query/Params",
            query_params_request,
            QueryParamsResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    async def eth_call(
        self,
        eth_call_request: "EthCallRequest",
        *,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None
    ) -> "MsgEthereumTxResponse":
        return await self._unary_unary(
            "/ethermint.evm.v1.Query/EthCall",
            eth_call_request,
            MsgEthereumTxResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    async def estimate_gas(
        self,
        eth_call_request: "EthCallRequest",
        *,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None
    ) -> "EstimateGasResponse":
        return await self._unary_unary(
            "/ethermint.evm.v1.Query/EstimateGas",
            eth_call_request,
            EstimateGasResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    async def trace_tx(
        self,
        query_trace_tx_request: "QueryTraceTxRequest",
        *,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None
    ) -> "QueryTraceTxResponse":
        return await self._unary_unary(
            "/ethermint.evm.v1.Query/TraceTx",
            query_trace_tx_request,
            QueryTraceTxResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    async def trace_block(
        self,
        query_trace_block_request: "QueryTraceBlockRequest",
        *,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None
    ) -> "QueryTraceBlockResponse":
        return await self._unary_unary(
            "/ethermint.evm.v1.Query/TraceBlock",
            query_trace_block_request,
            QueryTraceBlockResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    async def base_fee(
        self,
        query_base_fee_request: "QueryBaseFeeRequest",
        *,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None
    ) -> "QueryBaseFeeResponse":
        return await self._unary_unary(
            "/ethermint.evm.v1.Query/BaseFee",
            query_base_fee_request,
            QueryBaseFeeResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )


class MsgBase(ServiceBase):
    async def ethereum_tx(
        self, msg_ethereum_tx: "MsgEthereumTx"
    ) -> "MsgEthereumTxResponse":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def __rpc_ethereum_tx(
        self, stream: "grpclib.server.Stream[MsgEthereumTx, MsgEthereumTxResponse]"
    ) -> None:
        request = await stream.recv_message()
        response = await self.ethereum_tx(request)
        await stream.send_message(response)

    def __mapping__(self) -> Dict[str, grpclib.const.Handler]:
        return {
            "/ethermint.evm.v1.Msg/EthereumTx": grpclib.const.Handler(
                self.__rpc_ethereum_tx,
                grpclib.const.Cardinality.UNARY_UNARY,
                MsgEthereumTx,
                MsgEthereumTxResponse,
            ),
        }


class QueryBase(ServiceBase):
    async def account(
        self, query_account_request: "QueryAccountRequest"
    ) -> "QueryAccountResponse":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def cosmos_account(
        self, query_cosmos_account_request: "QueryCosmosAccountRequest"
    ) -> "QueryCosmosAccountResponse":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def validator_account(
        self, query_validator_account_request: "QueryValidatorAccountRequest"
    ) -> "QueryValidatorAccountResponse":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def balance(
        self, query_balance_request: "QueryBalanceRequest"
    ) -> "QueryBalanceResponse":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def storage(
        self, query_storage_request: "QueryStorageRequest"
    ) -> "QueryStorageResponse":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def code(self, query_code_request: "QueryCodeRequest") -> "QueryCodeResponse":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def params(
        self, query_params_request: "QueryParamsRequest"
    ) -> "QueryParamsResponse":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def eth_call(
        self, eth_call_request: "EthCallRequest"
    ) -> "MsgEthereumTxResponse":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def estimate_gas(
        self, eth_call_request: "EthCallRequest"
    ) -> "EstimateGasResponse":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def trace_tx(
        self, query_trace_tx_request: "QueryTraceTxRequest"
    ) -> "QueryTraceTxResponse":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def trace_block(
        self, query_trace_block_request: "QueryTraceBlockRequest"
    ) -> "QueryTraceBlockResponse":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def base_fee(
        self, query_base_fee_request: "QueryBaseFeeRequest"
    ) -> "QueryBaseFeeResponse":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def __rpc_account(
        self, stream: "grpclib.server.Stream[QueryAccountRequest, QueryAccountResponse]"
    ) -> None:
        request = await stream.recv_message()
        response = await self.account(request)
        await stream.send_message(response)

    async def __rpc_cosmos_account(
        self,
        stream: "grpclib.server.Stream[QueryCosmosAccountRequest, QueryCosmosAccountResponse]",
    ) -> None:
        request = await stream.recv_message()
        response = await self.cosmos_account(request)
        await stream.send_message(response)

    async def __rpc_validator_account(
        self,
        stream: "grpclib.server.Stream[QueryValidatorAccountRequest, QueryValidatorAccountResponse]",
    ) -> None:
        request = await stream.recv_message()
        response = await self.validator_account(request)
        await stream.send_message(response)

    async def __rpc_balance(
        self, stream: "grpclib.server.Stream[QueryBalanceRequest, QueryBalanceResponse]"
    ) -> None:
        request = await stream.recv_message()
        response = await self.balance(request)
        await stream.send_message(response)

    async def __rpc_storage(
        self, stream: "grpclib.server.Stream[QueryStorageRequest, QueryStorageResponse]"
    ) -> None:
        request = await stream.recv_message()
        response = await self.storage(request)
        await stream.send_message(response)

    async def __rpc_code(
        self, stream: "grpclib.server.Stream[QueryCodeRequest, QueryCodeResponse]"
    ) -> None:
        request = await stream.recv_message()
        response = await self.code(request)
        await stream.send_message(response)

    async def __rpc_params(
        self, stream: "grpclib.server.Stream[QueryParamsRequest, QueryParamsResponse]"
    ) -> None:
        request = await stream.recv_message()
        response = await self.params(request)
        await stream.send_message(response)

    async def __rpc_eth_call(
        self, stream: "grpclib.server.Stream[EthCallRequest, MsgEthereumTxResponse]"
    ) -> None:
        request = await stream.recv_message()
        response = await self.eth_call(request)
        await stream.send_message(response)

    async def __rpc_estimate_gas(
        self, stream: "grpclib.server.Stream[EthCallRequest, EstimateGasResponse]"
    ) -> None:
        request = await stream.recv_message()
        response = await self.estimate_gas(request)
        await stream.send_message(response)

    async def __rpc_trace_tx(
        self, stream: "grpclib.server.Stream[QueryTraceTxRequest, QueryTraceTxResponse]"
    ) -> None:
        request = await stream.recv_message()
        response = await self.trace_tx(request)
        await stream.send_message(response)

    async def __rpc_trace_block(
        self,
        stream: "grpclib.server.Stream[QueryTraceBlockRequest, QueryTraceBlockResponse]",
    ) -> None:
        request = await stream.recv_message()
        response = await self.trace_block(request)
        await stream.send_message(response)

    async def __rpc_base_fee(
        self, stream: "grpclib.server.Stream[QueryBaseFeeRequest, QueryBaseFeeResponse]"
    ) -> None:
        request = await stream.recv_message()
        response = await self.base_fee(request)
        await stream.send_message(response)

    def __mapping__(self) -> Dict[str, grpclib.const.Handler]:
        return {
            "/ethermint.evm.v1.Query/Account": grpclib.const.Handler(
                self.__rpc_account,
                grpclib.const.Cardinality.UNARY_UNARY,
                QueryAccountRequest,
                QueryAccountResponse,
            ),
            "/ethermint.evm.v1.Query/CosmosAccount": grpclib.const.Handler(
                self.__rpc_cosmos_account,
                grpclib.const.Cardinality.UNARY_UNARY,
                QueryCosmosAccountRequest,
                QueryCosmosAccountResponse,
            ),
            "/ethermint.evm.v1.Query/ValidatorAccount": grpclib.const.Handler(
                self.__rpc_validator_account,
                grpclib.const.Cardinality.UNARY_UNARY,
                QueryValidatorAccountRequest,
                QueryValidatorAccountResponse,
            ),
            "/ethermint.evm.v1.Query/Balance": grpclib.const.Handler(
                self.__rpc_balance,
                grpclib.const.Cardinality.UNARY_UNARY,
                QueryBalanceRequest,
                QueryBalanceResponse,
            ),
            "/ethermint.evm.v1.Query/Storage": grpclib.const.Handler(
                self.__rpc_storage,
                grpclib.const.Cardinality.UNARY_UNARY,
                QueryStorageRequest,
                QueryStorageResponse,
            ),
            "/ethermint.evm.v1.Query/Code": grpclib.const.Handler(
                self.__rpc_code,
                grpclib.const.Cardinality.UNARY_UNARY,
                QueryCodeRequest,
                QueryCodeResponse,
            ),
            "/ethermint.evm.v1.Query/Params": grpclib.const.Handler(
                self.__rpc_params,
                grpclib.const.Cardinality.UNARY_UNARY,
                QueryParamsRequest,
                QueryParamsResponse,
            ),
            "/ethermint.evm.v1.Query/EthCall": grpclib.const.Handler(
                self.__rpc_eth_call,
                grpclib.const.Cardinality.UNARY_UNARY,
                EthCallRequest,
                MsgEthereumTxResponse,
            ),
            "/ethermint.evm.v1.Query/EstimateGas": grpclib.const.Handler(
                self.__rpc_estimate_gas,
                grpclib.const.Cardinality.UNARY_UNARY,
                EthCallRequest,
                EstimateGasResponse,
            ),
            "/ethermint.evm.v1.Query/TraceTx": grpclib.const.Handler(
                self.__rpc_trace_tx,
                grpclib.const.Cardinality.UNARY_UNARY,
                QueryTraceTxRequest,
                QueryTraceTxResponse,
            ),
            "/ethermint.evm.v1.Query/TraceBlock": grpclib.const.Handler(
                self.__rpc_trace_block,
                grpclib.const.Cardinality.UNARY_UNARY,
                QueryTraceBlockRequest,
                QueryTraceBlockResponse,
            ),
            "/ethermint.evm.v1.Query/BaseFee": grpclib.const.Handler(
                self.__rpc_base_fee,
                grpclib.const.Cardinality.UNARY_UNARY,
                QueryBaseFeeRequest,
                QueryBaseFeeResponse,
            ),
        }



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# sources: glitterchain/consumer/consumer.proto, glitterchain/consumer/genesis.proto, glitterchain/consumer/params.proto, glitterchain/consumer/query.proto, glitterchain/consumer/tx.proto
# plugin: python-betterproto
# This file has been @generated

from dataclasses import dataclass
from datetime import (
    datetime,
    timedelta,
)
from typing import (
    TYPE_CHECKING,
    Dict,
    List,
    Optional,
)

import betterproto
import grpclib
from betterproto.grpc.grpclib_server import ServiceBase

from ...cosmos.base.query import v1beta1 as __cosmos_base_query_v1_beta1__


if TYPE_CHECKING:
    import grpclib.server
    from betterproto.grpc.grpclib_client import MetadataLike
    from grpclib.metadata import Deadline


@dataclass(eq=False, repr=False)
class PledgeRequest(betterproto.Message):
    from_address: str = betterproto.string_field(1)
    dataset_name: str = betterproto.string_field(2)
    amount: str = betterproto.string_field(3)


@dataclass(eq=False, repr=False)
class PledgeResponse(betterproto.Message):
    pass


@dataclass(eq=False, repr=False)
class ReleasePledgeRequest(betterproto.Message):
    from_address: str = betterproto.string_field(1)
    dataset_name: str = betterproto.string_field(2)
    amount: str = betterproto.string_field(3)


@dataclass(eq=False, repr=False)
class ReleasePledgeResponse(betterproto.Message):
    completion_time: datetime = betterproto.message_field(1)


@dataclass(eq=False, repr=False)
class Cpdt(betterproto.Message):
    """consumer pledge dataset token"""

    dataset_name: str = betterproto.string_field(1)
    amount: str = betterproto.string_field(2)


@dataclass(eq=False, repr=False)
class Consumer(betterproto.Message):
    consumer_address: str = betterproto.string_field(1)
    cpd_ts: List["Cpdt"] = betterproto.message_field(2)
    description: str = betterproto.string_field(3)


@dataclass(eq=False, repr=False)
class ReleasingCpdt(betterproto.Message):
    consumer_address: str = betterproto.string_field(1)
    dataset_name: str = betterproto.string_field(2)
    entries: List["ReleasingCpdtEntry"] = betterproto.message_field(3)


@dataclass(eq=False, repr=False)
class ReleasingCpdTs(betterproto.Message):
    r_cpd_ts: List["ReleasingCpdt"] = betterproto.message_field(1)


@dataclass(eq=False, repr=False)
class ReleasingCpdtEntry(betterproto.Message):
    creation_height: int = betterproto.int64_field(1)
    completion_time: datetime = betterproto.message_field(2)
    amount: str = betterproto.string_field(3)


@dataclass(eq=False, repr=False)
class ReleasingCpdtEvents(betterproto.Message):
    """*event*"""

    dataset_name: str = betterproto.string_field(1)
    consumer_address: str = betterproto.string_field(2)
    coins: str = betterproto.string_field(3)


@dataclass(eq=False, repr=False)
class ReleasingDatasetExpiredEvents(betterproto.Message):
    dataset_name: str = betterproto.string_field(1)
    consumer_address: str = betterproto.string_field(2)
    coins: str = betterproto.string_field(3)


@dataclass(eq=False, repr=False)
class Params(betterproto.Message):
    releasing_cpdt_wait_time: timedelta = betterproto.message_field(1)
    max_cpdt_entries: int = betterproto.uint32_field(2)


@dataclass(eq=False, repr=False)
class QueryParamsRequest(betterproto.Message):
    pass


@dataclass(eq=False, repr=False)
class QueryParamsResponse(betterproto.Message):
    params: "Params" = betterproto.message_field(1)
    """params holds all the parameters of this module."""


@dataclass(eq=False, repr=False)
class QueryConsumerRequest(betterproto.Message):
    address: str = betterproto.string_field(1)


@dataclass(eq=False, repr=False)
class QueryConsumerResponse(betterproto.Message):
    consumer: "Consumer" = betterproto.message_field(1)


@dataclass(eq=False, repr=False)
class QueryConsumersRequest(betterproto.Message):
    pagination: "__cosmos_base_query_v1_beta1__.PageRequest" = (
        betterproto.message_field(1)
    )


@dataclass(eq=False, repr=False)
class QueryConsumersResponse(betterproto.Message):
    consumer: List["Consumer"] = betterproto.message_field(1)
    pagination: "__cosmos_base_query_v1_beta1__.PageResponse" = (
        betterproto.message_field(2)
    )


@dataclass(eq=False, repr=False)
class QueryReleasingCpdtRequest(betterproto.Message):
    address: str = betterproto.string_field(1)


@dataclass(eq=False, repr=False)
class QueryReleasingCpdtResponse(betterproto.Message):
    releasing_cpd_ts: List["ReleasingCpdt"] = betterproto.message_field(1)


@dataclass(eq=False, repr=False)
class QueryReleasingCpdTsRequest(betterproto.Message):
    pagination: "__cosmos_base_query_v1_beta1__.PageRequest" = (
        betterproto.message_field(1)
    )


@dataclass(eq=False, repr=False)
class QueryReleasingCpdTsResponse(betterproto.Message):
    releasing_cpd_ts: List["ReleasingCpdt"] = betterproto.message_field(1)
    pagination: "__cosmos_base_query_v1_beta1__.PageResponse" = (
        betterproto.message_field(2)
    )


@dataclass(eq=False, repr=False)
class GenesisState(betterproto.Message):
    """GenesisState defines the index module's genesis state."""

    params: "Params" = betterproto.message_field(1)
    consumers: List["Consumer"] = betterproto.message_field(2)
    releasing_cpd_ts: List["ReleasingCpdt"] = betterproto.message_field(3)


class MsgStub(betterproto.ServiceStub):
    async def pledge(
        self,
        pledge_request: "PledgeRequest",
        *,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None
    ) -> "PledgeResponse":
        return await self._unary_unary(
            "/glitterchain.consumer.Msg/Pledge",
            pledge_request,
            PledgeResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    async def release_pledge(
        self,
        release_pledge_request: "ReleasePledgeRequest",
        *,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None
    ) -> "ReleasePledgeResponse":
        return await self._unary_unary(
            "/glitterchain.consumer.Msg/ReleasePledge",
            release_pledge_request,
            ReleasePledgeResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )


class QueryStub(betterproto.ServiceStub):
    async def params(
        self,
        query_params_request: "QueryParamsRequest",
        *,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None
    ) -> "QueryParamsResponse":
        return await self._unary_unary(
            "/glitterchain.consumer.Query/Params",
            query_params_request,
            QueryParamsResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    async def query_consumer(
        self,
        query_consumer_request: "QueryConsumerRequest",
        *,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None
    ) -> "QueryConsumerResponse":
        return await self._unary_unary(
            "/glitterchain.consumer.Query/QueryConsumer",
            query_consumer_request,
            QueryConsumerResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    async def query_consumers(
        self,
        query_consumers_request: "QueryConsumersRequest",
        *,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None
    ) -> "QueryConsumersResponse":
        return await self._unary_unary(
            "/glitterchain.consumer.Query/QueryConsumers",
            query_consumers_request,
            QueryConsumersResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    async def query_releasing_cpdt(
        self,
        query_releasing_cpdt_request: "QueryReleasingCpdtRequest",
        *,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None
    ) -> "QueryReleasingCpdtResponse":
        return await self._unary_unary(
            "/glitterchain.consumer.Query/QueryReleasingCPDT",
            query_releasing_cpdt_request,
            QueryReleasingCpdtResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    async def query_releasing_cpd_ts(
        self,
        query_releasing_cpd_ts_request: "QueryReleasingCpdTsRequest",
        *,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None
    ) -> "QueryReleasingCpdTsResponse":
        return await self._unary_unary(
            "/glitterchain.consumer.Query/QueryReleasingCPDTs",
            query_releasing_cpd_ts_request,
            QueryReleasingCpdTsResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )


class MsgBase(ServiceBase):
    async def pledge(self, pledge_request: "PledgeRequest") -> "PledgeResponse":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def release_pledge(
        self, release_pledge_request: "ReleasePledgeRequest"
    ) -> "ReleasePledgeResponse":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def __rpc_pledge(
        self, stream: "grpclib.server.Stream[PledgeRequest, PledgeResponse]"
    ) -> None:
        request = await stream.recv_message()
        response = await self.pledge(request)
        await stream.send_message(response)

    async def __rpc_release_pledge(
        self,
        stream: "grpclib.server.Stream[ReleasePledgeRequest, ReleasePledgeResponse]",
    ) -> None:
        request = await stream.recv_message()
        response = await self.release_pledge(request)
        await stream.send_message(response)

    def __mapping__(self) -> Dict[str, grpclib.const.Handler]:
        return {
            "/glitterchain.consumer.Msg/Pledge": grpclib.const.Handler(
                self.__rpc_pledge,
                grpclib.const.Cardinality.UNARY_UNARY,
                PledgeRequest,
                PledgeResponse,
            ),
            "/glitterchain.consumer.Msg/ReleasePledge": grpclib.const.Handler(
                self.__rpc_release_pledge,
                grpclib.const.Cardinality.UNARY_UNARY,
                ReleasePledgeRequest,
                ReleasePledgeResponse,
            ),
        }


class QueryBase(ServiceBase):
    async def params(
        self, query_params_request: "QueryParamsRequest"
    ) -> "QueryParamsResponse":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def query_consumer(
        self, query_consumer_request: "QueryConsumerRequest"
    ) -> "QueryConsumerResponse":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def query_consumers(
        self, query_consumers_request: "QueryConsumersRequest"
    ) -> "QueryConsumersResponse":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def query_releasing_cpdt(
        self, query_releasing_cpdt_request: "QueryReleasingCpdtRequest"
    ) -> "QueryReleasingCpdtResponse":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def query_releasing_cpd_ts(
        self, query_releasing_cpd_ts_request: "QueryReleasingCpdTsRequest"
    ) -> "QueryReleasingCpdTsResponse":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def __rpc_params(
        self, stream: "grpclib.server.Stream[QueryParamsRequest, QueryParamsResponse]"
    ) -> None:
        request = await stream.recv_message()
        response = await self.params(request)
        await stream.send_message(response)

    async def __rpc_query_consumer(
        self,
        stream: "grpclib.server.Stream[QueryConsumerRequest, QueryConsumerResponse]",
    ) -> None:
        request = await stream.recv_message()
        response = await self.query_consumer(request)
        await stream.send_message(response)

    async def __rpc_query_consumers(
        self,
        stream: "grpclib.server.Stream[QueryConsumersRequest, QueryConsumersResponse]",
    ) -> None:
        request = await stream.recv_message()
        response = await self.query_consumers(request)
        await stream.send_message(response)

    async def __rpc_query_releasing_cpdt(
        self,
        stream: "grpclib.server.Stream[QueryReleasingCpdtRequest, QueryReleasingCpdtResponse]",
    ) -> None:
        request = await stream.recv_message()
        response = await self.query_releasing_cpdt(request)
        await stream.send_message(response)

    async def __rpc_query_releasing_cpd_ts(
        self,
        stream: "grpclib.server.Stream[QueryReleasingCpdTsRequest, QueryReleasingCpdTsResponse]",
    ) -> None:
        request = await stream.recv_message()
        response = await self.query_releasing_cpd_ts(request)
        await stream.send_message(response)

    def __mapping__(self) -> Dict[str, grpclib.const.Handler]:
        return {
            "/glitterchain.consumer.Query/Params": grpclib.const.Handler(
                self.__rpc_params,
                grpclib.const.Cardinality.UNARY_UNARY,
                QueryParamsRequest,
                QueryParamsResponse,
            ),
            "/glitterchain.consumer.Query/QueryConsumer": grpclib.const.Handler(
                self.__rpc_query_consumer,
                grpclib.const.Cardinality.UNARY_UNARY,
                QueryConsumerRequest,
                QueryConsumerResponse,
            ),
            "/glitterchain.consumer.Query/QueryConsumers": grpclib.const.Handler(
                self.__rpc_query_consumers,
                grpclib.const.Cardinality.UNARY_UNARY,
                QueryConsumersRequest,
                QueryConsumersResponse,
            ),
            "/glitterchain.consumer.Query/QueryReleasingCPDT": grpclib.const.Handler(
                self.__rpc_query_releasing_cpdt,
                grpclib.const.Cardinality.UNARY_UNARY,
                QueryReleasingCpdtRequest,
                QueryReleasingCpdtResponse,
            ),
            "/glitterchain.consumer.Query/QueryReleasingCPDTs": grpclib.const.Handler(
                self.__rpc_query_releasing_cpd_ts,
                grpclib.const.Cardinality.UNARY_UNARY,
                QueryReleasingCpdTsRequest,
                QueryReleasingCpdTsResponse,
            ),
        }



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# sources: ibc/core/client/v1/client.proto, ibc/core/client/v1/genesis.proto, ibc/core/client/v1/query.proto, ibc/core/client/v1/tx.proto
# plugin: python-betterproto
# This file has been @generated

from dataclasses import dataclass
from typing import (
    TYPE_CHECKING,
    Dict,
    List,
    Optional,
)

import betterproto
import betterproto.lib.google.protobuf as betterproto_lib_google_protobuf
import grpclib
from betterproto.grpc.grpclib_server import ServiceBase

from .....cosmos.base.query import v1beta1 as ____cosmos_base_query_v1_beta1__
from .....cosmos.upgrade import v1beta1 as ____cosmos_upgrade_v1_beta1__


if TYPE_CHECKING:
    import grpclib.server
    from betterproto.grpc.grpclib_client import MetadataLike
    from grpclib.metadata import Deadline


@dataclass(eq=False, repr=False)
class IdentifiedClientState(betterproto.Message):
    """
    IdentifiedClientState defines a client state with an additional client
     identifier field.
    """

    client_id: str = betterproto.string_field(1)
    """client identifier"""

    client_state: "betterproto_lib_google_protobuf.Any" = betterproto.message_field(2)
    """client state"""


@dataclass(eq=False, repr=False)
class ConsensusStateWithHeight(betterproto.Message):
    """
    ConsensusStateWithHeight defines a consensus state with an additional height
     field.
    """

    height: "Height" = betterproto.message_field(1)
    """consensus state height"""

    consensus_state: "betterproto_lib_google_protobuf.Any" = betterproto.message_field(
        2
    )
    """consensus state"""


@dataclass(eq=False, repr=False)
class ClientConsensusStates(betterproto.Message):
    """
    ClientConsensusStates defines all the stored consensus states for a given
     client.
    """

    client_id: str = betterproto.string_field(1)
    """client identifier"""

    consensus_states: List["ConsensusStateWithHeight"] = betterproto.message_field(2)
    """consensus states and their heights associated with the client"""


@dataclass(eq=False, repr=False)
class ClientUpdateProposal(betterproto.Message):
    """
    ClientUpdateProposal is a governance proposal. If it passes, the substitute
     client's latest consensus state is copied over to the subject client. The proposal
     handler may fail if the subject and the substitute do not match in client and
     chain parameters (with exception to latest height, frozen height, and chain-id).
    """

    title: str = betterproto.string_field(1)
    """the title of the update proposal"""

    description: str = betterproto.string_field(2)
    """the description of the proposal"""

    subject_client_id: str = betterproto.string_field(3)
    """
    the client identifier for the client to be updated if the proposal passes
    """

    substitute_client_id: str = betterproto.string_field(4)
    """
    the substitute client identifier for the client standing in for the subject
     client
    """


@dataclass(eq=False, repr=False)
class UpgradeProposal(betterproto.Message):
    """
    UpgradeProposal is a gov Content type for initiating an IBC breaking
     upgrade.
    """

    title: str = betterproto.string_field(1)
    description: str = betterproto.string_field(2)
    plan: "____cosmos_upgrade_v1_beta1__.Plan" = betterproto.message_field(3)
    upgraded_client_state: "betterproto_lib_google_protobuf.Any" = (
        betterproto.message_field(4)
    )
    """
    An UpgradedClientState must be provided to perform an IBC breaking upgrade.
     This will make the chain commit to the correct upgraded (self) client state
     before the upgrade occurs, so that connecting chains can verify that the
     new upgraded client is valid by verifying a proof on the previous version
     of the chain. This will allow IBC connections to persist smoothly across
     planned chain upgrades
    """


@dataclass(eq=False, repr=False)
class Height(betterproto.Message):
    """
    Height is a monotonically increasing data type
     that can be compared against another Height for the purposes of updating and
     freezing clients

     Normally the RevisionHeight is incremented at each height while keeping
     RevisionNumber the same. However some consensus algorithms may choose to
     reset the height in certain conditions e.g. hard forks, state-machine
     breaking changes In these cases, the RevisionNumber is incremented so that
     height continues to be monitonically increasing even as the RevisionHeight
     gets reset
    """

    revision_number: int = betterproto.uint64_field(1)
    """the revision that the client is currently on"""

    revision_height: int = betterproto.uint64_field(2)
    """the height within the given revision"""


@dataclass(eq=False, repr=False)
class Params(betterproto.Message):
    """Params defines the set of IBC light client parameters."""

    allowed_clients: List[str] = betterproto.string_field(1)
    """allowed_clients defines the list of allowed client state types."""


@dataclass(eq=False, repr=False)
class GenesisState(betterproto.Message):
    """GenesisState defines the ibc client submodule's genesis state."""

    clients: List["IdentifiedClientState"] = betterproto.message_field(1)
    """client states with their corresponding identifiers"""

    clients_consensus: List["ClientConsensusStates"] = betterproto.message_field(2)
    """consensus states from each client"""

    clients_metadata: List["IdentifiedGenesisMetadata"] = betterproto.message_field(3)
    """metadata from each client"""

    params: "Params" = betterproto.message_field(4)
    create_localhost: bool = betterproto.bool_field(5)
    """create localhost on initialization"""

    next_client_sequence: int = betterproto.uint64_field(6)
    """the sequence for the next generated client identifier"""


@dataclass(eq=False, repr=False)
class GenesisMetadata(betterproto.Message):
    """
    GenesisMetadata defines the genesis type for metadata that clients may return
     with ExportMetadata
    """

    key: bytes = betterproto.bytes_field(1)
    """store key of metadata without clientID-prefix"""

    value: bytes = betterproto.bytes_field(2)
    """metadata value"""


@dataclass(eq=False, repr=False)
class IdentifiedGenesisMetadata(betterproto.Message):
    """
    IdentifiedGenesisMetadata has the client metadata with the corresponding
     client id.
    """

    client_id: str = betterproto.string_field(1)
    client_metadata: List["GenesisMetadata"] = betterproto.message_field(2)


@dataclass(eq=False, repr=False)
class MsgCreateClient(betterproto.Message):
    """MsgCreateClient defines a message to create an IBC client"""

    client_state: "betterproto_lib_google_protobuf.Any" = betterproto.message_field(1)
    """light client state"""

    consensus_state: "betterproto_lib_google_protobuf.Any" = betterproto.message_field(
        2
    )
    """
    consensus state associated with the client that corresponds to a given
     height.
    """

    signer: str = betterproto.string_field(3)
    """signer address"""


@dataclass(eq=False, repr=False)
class MsgCreateClientResponse(betterproto.Message):
    """MsgCreateClientResponse defines the Msg/CreateClient response type."""

    pass


@dataclass(eq=False, repr=False)
class MsgUpdateClient(betterproto.Message):
    """
    MsgUpdateClient defines an sdk.Msg to update a IBC client state using
     the given header.
    """

    client_id: str = betterproto.string_field(1)
    """client unique identifier"""

    header: "betterproto_lib_google_protobuf.Any" = betterproto.message_field(2)
    """header to update the light client"""

    signer: str = betterproto.string_field(3)
    """signer address"""


@dataclass(eq=False, repr=False)
class MsgUpdateClientResponse(betterproto.Message):
    """MsgUpdateClientResponse defines the Msg/UpdateClient response type."""

    pass


@dataclass(eq=False, repr=False)
class MsgUpgradeClient(betterproto.Message):
    """
    MsgUpgradeClient defines an sdk.Msg to upgrade an IBC client to a new client
     state
    """

    client_id: str = betterproto.string_field(1)
    """client unique identifier"""

    client_state: "betterproto_lib_google_protobuf.Any" = betterproto.message_field(2)
    """upgraded client state"""

    consensus_state: "betterproto_lib_google_protobuf.Any" = betterproto.message_field(
        3
    )
    """
    upgraded consensus state, only contains enough information to serve as a
     basis of trust in update logic
    """

    proof_upgrade_client: bytes = betterproto.bytes_field(4)
    """proof that old chain committed to new client"""

    proof_upgrade_consensus_state: bytes = betterproto.bytes_field(5)
    """proof that old chain committed to new consensus state"""

    signer: str = betterproto.string_field(6)
    """signer address"""


@dataclass(eq=False, repr=False)
class MsgUpgradeClientResponse(betterproto.Message):
    """
    MsgUpgradeClientResponse defines the Msg/UpgradeClient response type.
    """

    pass


@dataclass(eq=False, repr=False)
class MsgSubmitMisbehaviour(betterproto.Message):
    """
    MsgSubmitMisbehaviour defines an sdk.Msg type that submits Evidence for
     light client misbehaviour.
    """

    client_id: str = betterproto.string_field(1)
    """client unique identifier"""

    misbehaviour: "betterproto_lib_google_protobuf.Any" = betterproto.message_field(2)
    """misbehaviour used for freezing the light client"""

    signer: str = betterproto.string_field(3)
    """signer address"""


@dataclass(eq=False, repr=False)
class MsgSubmitMisbehaviourResponse(betterproto.Message):
    """
    MsgSubmitMisbehaviourResponse defines the Msg/SubmitMisbehaviour response
     type.
    """

    pass


@dataclass(eq=False, repr=False)
class QueryClientStateRequest(betterproto.Message):
    """
    QueryClientStateRequest is the request type for the Query/ClientState RPC
     method
    """

    client_id: str = betterproto.string_field(1)
    """client state unique identifier"""


@dataclass(eq=False, repr=False)
class QueryClientStateResponse(betterproto.Message):
    """
    QueryClientStateResponse is the response type for the Query/ClientState RPC
     method. Besides the client state, it includes a proof and the height from
     which the proof was retrieved.
    """

    client_state: "betterproto_lib_google_protobuf.Any" = betterproto.message_field(1)
    """client state associated with the request identifier"""

    proof: bytes = betterproto.bytes_field(2)
    """merkle proof of existence"""

    proof_height: "Height" = betterproto.message_field(3)
    """height at which the proof was retrieved"""


@dataclass(eq=False, repr=False)
class QueryClientStatesRequest(betterproto.Message):
    """
    QueryClientStatesRequest is the request type for the Query/ClientStates RPC
     method
    """

    pagination: "____cosmos_base_query_v1_beta1__.PageRequest" = (
        betterproto.message_field(1)
    )
    """pagination request"""


@dataclass(eq=False, repr=False)
class QueryClientStatesResponse(betterproto.Message):
    """
    QueryClientStatesResponse is the response type for the Query/ClientStates RPC
     method.
    """

    client_states: List["IdentifiedClientState"] = betterproto.message_field(1)
    """list of stored ClientStates of the chain."""

    pagination: "____cosmos_base_query_v1_beta1__.PageResponse" = (
        betterproto.message_field(2)
    )
    """pagination response"""


@dataclass(eq=False, repr=False)
class QueryConsensusStateRequest(betterproto.Message):
    """
    QueryConsensusStateRequest is the request type for the Query/ConsensusState
     RPC method. Besides the consensus state, it includes a proof and the height
     from which the proof was retrieved.
    """

    client_id: str = betterproto.string_field(1)
    """client identifier"""

    revision_number: int = betterproto.uint64_field(2)
    """consensus state revision number"""

    revision_height: int = betterproto.uint64_field(3)
    """consensus state revision height"""

    latest_height: bool = betterproto.bool_field(4)
    """
    latest_height overrrides the height field and queries the latest stored
     ConsensusState
    """


@dataclass(eq=False, repr=False)
class QueryConsensusStateResponse(betterproto.Message):
    """
    QueryConsensusStateResponse is the response type for the Query/ConsensusState
     RPC method
    """

    consensus_state: "betterproto_lib_google_protobuf.Any" = betterproto.message_field(
        1
    )
    """
    consensus state associated with the client identifier at the given height
    """

    proof: bytes = betterproto.bytes_field(2)
    """merkle proof of existence"""

    proof_height: "Height" = betterproto.message_field(3)
    """height at which the proof was retrieved"""


@dataclass(eq=False, repr=False)
class QueryConsensusStatesRequest(betterproto.Message):
    """
    QueryConsensusStatesRequest is the request type for the Query/ConsensusStates
     RPC method.
    """

    client_id: str = betterproto.string_field(1)
    """client identifier"""

    pagination: "____cosmos_base_query_v1_beta1__.PageRequest" = (
        betterproto.message_field(2)
    )
    """pagination request"""


@dataclass(eq=False, repr=False)
class QueryConsensusStatesResponse(betterproto.Message):
    """
    QueryConsensusStatesResponse is the response type for the
     Query/ConsensusStates RPC method
    """

    consensus_states: List["ConsensusStateWithHeight"] = betterproto.message_field(1)
    """consensus states associated with the identifier"""

    pagination: "____cosmos_base_query_v1_beta1__.PageResponse" = (
        betterproto.message_field(2)
    )
    """pagination response"""


@dataclass(eq=False, repr=False)
class QueryClientStatusRequest(betterproto.Message):
    """
    QueryClientStatusRequest is the request type for the Query/ClientStatus RPC
     method
    """

    client_id: str = betterproto.string_field(1)
    """client unique identifier"""


@dataclass(eq=False, repr=False)
class QueryClientStatusResponse(betterproto.Message):
    """
    QueryClientStatusResponse is the response type for the Query/ClientStatus RPC
     method. It returns the current status of the IBC client.
    """

    status: str = betterproto.string_field(1)


@dataclass(eq=False, repr=False)
class QueryClientParamsRequest(betterproto.Message):
    """
    QueryClientParamsRequest is the request type for the Query/ClientParams RPC
     method.
    """

    pass


@dataclass(eq=False, repr=False)
class QueryClientParamsResponse(betterproto.Message):
    """
    QueryClientParamsResponse is the response type for the Query/ClientParams RPC
     method.
    """

    params: "Params" = betterproto.message_field(1)
    """params defines the parameters of the module."""


@dataclass(eq=False, repr=False)
class QueryUpgradedClientStateRequest(betterproto.Message):
    """
    QueryUpgradedClientStateRequest is the request type for the
     Query/UpgradedClientState RPC method
    """

    pass


@dataclass(eq=False, repr=False)
class QueryUpgradedClientStateResponse(betterproto.Message):
    """
    QueryUpgradedClientStateResponse is the response type for the
     Query/UpgradedClientState RPC method.
    """

    upgraded_client_state: "betterproto_lib_google_protobuf.Any" = (
        betterproto.message_field(1)
    )
    """client state associated with the request identifier"""


@dataclass(eq=False, repr=False)
class QueryUpgradedConsensusStateRequest(betterproto.Message):
    """
    QueryUpgradedConsensusStateRequest is the request type for the
     Query/UpgradedConsensusState RPC method
    """

    pass


@dataclass(eq=False, repr=False)
class QueryUpgradedConsensusStateResponse(betterproto.Message):
    """
    QueryUpgradedConsensusStateResponse is the response type for the
     Query/UpgradedConsensusState RPC method.
    """

    upgraded_consensus_state: "betterproto_lib_google_protobuf.Any" = (
        betterproto.message_field(1)
    )
    """Consensus state associated with the request identifier"""


class MsgStub(betterproto.ServiceStub):
    async def create_client(
        self,
        msg_create_client: "MsgCreateClient",
        *,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None
    ) -> "MsgCreateClientResponse":
        return await self._unary_unary(
            "/ibc.core.client.v1.Msg/CreateClient",
            msg_create_client,
            MsgCreateClientResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    async def update_client(
        self,
        msg_update_client: "MsgUpdateClient",
        *,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None
    ) -> "MsgUpdateClientResponse":
        return await self._unary_unary(
            "/ibc.core.client.v1.Msg/UpdateClient",
            msg_update_client,
            MsgUpdateClientResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    async def upgrade_client(
        self,
        msg_upgrade_client: "MsgUpgradeClient",
        *,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None
    ) -> "MsgUpgradeClientResponse":
        return await self._unary_unary(
            "/ibc.core.client.v1.Msg/UpgradeClient",
            msg_upgrade_client,
            MsgUpgradeClientResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    async def submit_misbehaviour(
        self,
        msg_submit_misbehaviour: "MsgSubmitMisbehaviour",
        *,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None
    ) -> "MsgSubmitMisbehaviourResponse":
        return await self._unary_unary(
            "/ibc.core.client.v1.Msg/SubmitMisbehaviour",
            msg_submit_misbehaviour,
            MsgSubmitMisbehaviourResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )


class QueryStub(betterproto.ServiceStub):
    async def client_state(
        self,
        query_client_state_request: "QueryClientStateRequest",
        *,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None
    ) -> "QueryClientStateResponse":
        return await self._unary_unary(
            "/ibc.core.client.v1.Query/ClientState",
            query_client_state_request,
            QueryClientStateResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    async def client_states(
        self,
        query_client_states_request: "QueryClientStatesRequest",
        *,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None
    ) -> "QueryClientStatesResponse":
        return await self._unary_unary(
            "/ibc.core.client.v1.Query/ClientStates",
            query_client_states_request,
            QueryClientStatesResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    async def consensus_state(
        self,
        query_consensus_state_request: "QueryConsensusStateRequest",
        *,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None
    ) -> "QueryConsensusStateResponse":
        return await self._unary_unary(
            "/ibc.core.client.v1.Query/ConsensusState",
            query_consensus_state_request,
            QueryConsensusStateResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    async def consensus_states(
        self,
        query_consensus_states_request: "QueryConsensusStatesRequest",
        *,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None
    ) -> "QueryConsensusStatesResponse":
        return await self._unary_unary(
            "/ibc.core.client.v1.Query/ConsensusStates",
            query_consensus_states_request,
            QueryConsensusStatesResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    async def client_status(
        self,
        query_client_status_request: "QueryClientStatusRequest",
        *,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None
    ) -> "QueryClientStatusResponse":
        return await self._unary_unary(
            "/ibc.core.client.v1.Query/ClientStatus",
            query_client_status_request,
            QueryClientStatusResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    async def client_params(
        self,
        query_client_params_request: "QueryClientParamsRequest",
        *,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None
    ) -> "QueryClientParamsResponse":
        return await self._unary_unary(
            "/ibc.core.client.v1.Query/ClientParams",
            query_client_params_request,
            QueryClientParamsResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    async def upgraded_client_state(
        self,
        query_upgraded_client_state_request: "QueryUpgradedClientStateRequest",
        *,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None
    ) -> "QueryUpgradedClientStateResponse":
        return await self._unary_unary(
            "/ibc.core.client.v1.Query/UpgradedClientState",
            query_upgraded_client_state_request,
            QueryUpgradedClientStateResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    async def upgraded_consensus_state(
        self,
        query_upgraded_consensus_state_request: "QueryUpgradedConsensusStateRequest",
        *,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None
    ) -> "QueryUpgradedConsensusStateResponse":
        return await self._unary_unary(
            "/ibc.core.client.v1.Query/UpgradedConsensusState",
            query_upgraded_consensus_state_request,
            QueryUpgradedConsensusStateResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )


class MsgBase(ServiceBase):
    async def create_client(
        self, msg_create_client: "MsgCreateClient"
    ) -> "MsgCreateClientResponse":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def update_client(
        self, msg_update_client: "MsgUpdateClient"
    ) -> "MsgUpdateClientResponse":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def upgrade_client(
        self, msg_upgrade_client: "MsgUpgradeClient"
    ) -> "MsgUpgradeClientResponse":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def submit_misbehaviour(
        self, msg_submit_misbehaviour: "MsgSubmitMisbehaviour"
    ) -> "MsgSubmitMisbehaviourResponse":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def __rpc_create_client(
        self, stream: "grpclib.server.Stream[MsgCreateClient, MsgCreateClientResponse]"
    ) -> None:
        request = await stream.recv_message()
        response = await self.create_client(request)
        await stream.send_message(response)

    async def __rpc_update_client(
        self, stream: "grpclib.server.Stream[MsgUpdateClient, MsgUpdateClientResponse]"
    ) -> None:
        request = await stream.recv_message()
        response = await self.update_client(request)
        await stream.send_message(response)

    async def __rpc_upgrade_client(
        self,
        stream: "grpclib.server.Stream[MsgUpgradeClient, MsgUpgradeClientResponse]",
    ) -> None:
        request = await stream.recv_message()
        response = await self.upgrade_client(request)
        await stream.send_message(response)

    async def __rpc_submit_misbehaviour(
        self,
        stream: "grpclib.server.Stream[MsgSubmitMisbehaviour, MsgSubmitMisbehaviourResponse]",
    ) -> None:
        request = await stream.recv_message()
        response = await self.submit_misbehaviour(request)
        await stream.send_message(response)

    def __mapping__(self) -> Dict[str, grpclib.const.Handler]:
        return {
            "/ibc.core.client.v1.Msg/CreateClient": grpclib.const.Handler(
                self.__rpc_create_client,
                grpclib.const.Cardinality.UNARY_UNARY,
                MsgCreateClient,
                MsgCreateClientResponse,
            ),
            "/ibc.core.client.v1.Msg/UpdateClient": grpclib.const.Handler(
                self.__rpc_update_client,
                grpclib.const.Cardinality.UNARY_UNARY,
                MsgUpdateClient,
                MsgUpdateClientResponse,
            ),
            "/ibc.core.client.v1.Msg/UpgradeClient": grpclib.const.Handler(
                self.__rpc_upgrade_client,
                grpclib.const.Cardinality.UNARY_UNARY,
                MsgUpgradeClient,
                MsgUpgradeClientResponse,
            ),
            "/ibc.core.client.v1.Msg/SubmitMisbehaviour": grpclib.const.Handler(
                self.__rpc_submit_misbehaviour,
                grpclib.const.Cardinality.UNARY_UNARY,
                MsgSubmitMisbehaviour,
                MsgSubmitMisbehaviourResponse,
            ),
        }


class QueryBase(ServiceBase):
    async def client_state(
        self, query_client_state_request: "QueryClientStateRequest"
    ) -> "QueryClientStateResponse":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def client_states(
        self, query_client_states_request: "QueryClientStatesRequest"
    ) -> "QueryClientStatesResponse":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def consensus_state(
        self, query_consensus_state_request: "QueryConsensusStateRequest"
    ) -> "QueryConsensusStateResponse":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def consensus_states(
        self, query_consensus_states_request: "QueryConsensusStatesRequest"
    ) -> "QueryConsensusStatesResponse":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def client_status(
        self, query_client_status_request: "QueryClientStatusRequest"
    ) -> "QueryClientStatusResponse":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def client_params(
        self, query_client_params_request: "QueryClientParamsRequest"
    ) -> "QueryClientParamsResponse":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def upgraded_client_state(
        self, query_upgraded_client_state_request: "QueryUpgradedClientStateRequest"
    ) -> "QueryUpgradedClientStateResponse":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def upgraded_consensus_state(
        self,
        query_upgraded_consensus_state_request: "QueryUpgradedConsensusStateRequest",
    ) -> "QueryUpgradedConsensusStateResponse":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def __rpc_client_state(
        self,
        stream: "grpclib.server.Stream[QueryClientStateRequest, QueryClientStateResponse]",
    ) -> None:
        request = await stream.recv_message()
        response = await self.client_state(request)
        await stream.send_message(response)

    async def __rpc_client_states(
        self,
        stream: "grpclib.server.Stream[QueryClientStatesRequest, QueryClientStatesResponse]",
    ) -> None:
        request = await stream.recv_message()
        response = await self.client_states(request)
        await stream.send_message(response)

    async def __rpc_consensus_state(
        self,
        stream: "grpclib.server.Stream[QueryConsensusStateRequest, QueryConsensusStateResponse]",
    ) -> None:
        request = await stream.recv_message()
        response = await self.consensus_state(request)
        await stream.send_message(response)

    async def __rpc_consensus_states(
        self,
        stream: "grpclib.server.Stream[QueryConsensusStatesRequest, QueryConsensusStatesResponse]",
    ) -> None:
        request = await stream.recv_message()
        response = await self.consensus_states(request)
        await stream.send_message(response)

    async def __rpc_client_status(
        self,
        stream: "grpclib.server.Stream[QueryClientStatusRequest, QueryClientStatusResponse]",
    ) -> None:
        request = await stream.recv_message()
        response = await self.client_status(request)
        await stream.send_message(response)

    async def __rpc_client_params(
        self,
        stream: "grpclib.server.Stream[QueryClientParamsRequest, QueryClientParamsResponse]",
    ) -> None:
        request = await stream.recv_message()
        response = await self.client_params(request)
        await stream.send_message(response)

    async def __rpc_upgraded_client_state(
        self,
        stream: "grpclib.server.Stream[QueryUpgradedClientStateRequest, QueryUpgradedClientStateResponse]",
    ) -> None:
        request = await stream.recv_message()
        response = await self.upgraded_client_state(request)
        await stream.send_message(response)

    async def __rpc_upgraded_consensus_state(
        self,
        stream: "grpclib.server.Stream[QueryUpgradedConsensusStateRequest, QueryUpgradedConsensusStateResponse]",
    ) -> None:
        request = await stream.recv_message()
        response = await self.upgraded_consensus_state(request)
        await stream.send_message(response)

    def __mapping__(self) -> Dict[str, grpclib.const.Handler]:
        return {
            "/ibc.core.client.v1.Query/ClientState": grpclib.const.Handler(
                self.__rpc_client_state,
                grpclib.const.Cardinality.UNARY_UNARY,
                QueryClientStateRequest,
                QueryClientStateResponse,
            ),
            "/ibc.core.client.v1.Query/ClientStates": grpclib.const.Handler(
                self.__rpc_client_states,
                grpclib.const.Cardinality.UNARY_UNARY,
                QueryClientStatesRequest,
                QueryClientStatesResponse,
            ),
            "/ibc.core.client.v1.Query/ConsensusState": grpclib.const.Handler(
                self.__rpc_consensus_state,
                grpclib.const.Cardinality.UNARY_UNARY,
                QueryConsensusStateRequest,
                QueryConsensusStateResponse,
            ),
            "/ibc.core.client.v1.Query/ConsensusStates": grpclib.const.Handler(
                self.__rpc_consensus_states,
                grpclib.const.Cardinality.UNARY_UNARY,
                QueryConsensusStatesRequest,
                QueryConsensusStatesResponse,
            ),
            "/ibc.core.client.v1.Query/ClientStatus": grpclib.const.Handler(
                self.__rpc_client_status,
                grpclib.const.Cardinality.UNARY_UNARY,
                QueryClientStatusRequest,
                QueryClientStatusResponse,
            ),
            "/ibc.core.client.v1.Query/ClientParams": grpclib.const.Handler(
                self.__rpc_client_params,
                grpclib.const.Cardinality.UNARY_UNARY,
                QueryClientParamsRequest,
                QueryClientParamsResponse,
            ),
            "/ibc.core.client.v1.Query/UpgradedClientState": grpclib.const.Handler(
                self.__rpc_upgraded_client_state,
                grpclib.const.Cardinality.UNARY_UNARY,
                QueryUpgradedClientStateRequest,
                QueryUpgradedClientStateResponse,
            ),
            "/ibc.core.client.v1.Query/UpgradedConsensusState": grpclib.const.Handler(
                self.__rpc_upgraded_consensus_state,
                grpclib.const.Cardinality.UNARY_UNARY,
                QueryUpgradedConsensusStateRequest,
                QueryUpgradedConsensusStateResponse,
            ),
        }
